"""Module for getting full paths."""
from __future__ import annotations

import pathlib
import typing


def path(__path: str, /) -> pathlib.Path:
    """Return the full path of the given path."""
    return pathlib.Path(__path).expanduser().resolve()


def exists(__path: typing.Union[str, pathlib.Path], /) -> bool:
    """Return True if the given path exists."""
    return path(__path).exists()



"""Initialise the project directory with the required files."""
from __future__ import annotations

import jade.path as path
from jade.commands.initialise.files import FILES


def initialise():
    """Initialise the project directory with the required files."""
    for raw in FILES:
        file = path.path(raw)
        if path.exists(file):
            print(f"File already exists: {file}\nAborting...")
            exit(1)

        else:
            if not path.exists(file.parent):
                file.parent.mkdir()

            with open(path.path(file), "w") as f:
                f.write(FILES[raw].removeprefix("\n"))

    print("Done!")
    exit(0)



"""Initialise a new Jade project."""
from __future__ import annotations

import sys

import jade.commands


def main():
    """Run the Jade CLI."""
    commands = {
        "init": jade.commands.initialise,
    }

    try:
        command = sys.argv[1]
    except IndexError:
        print("Welcome to Jade.")
        print("Available commands: " + ", ".join(commands.keys()) + ".")
        exit(0)

    if command in commands:
        commands[command]()

    else:
        print(f"Unknown command: {command}")
        print("Available commands: " + ", ".join(commands.keys()) + ".")
        exit(1)


if __name__ == "__main__":
    main()



# flake8: noqa
"""Files to be created when initialising a new project.""" ""
from __future__ import annotations

FILES = {
    "main.py": '''
"""Run the default Jade app."""
from __future__ import annotations

import flask
import jade


app = flask.Flask(__name__)


@app.route("/", methods=["GET"])
def index() -> str:
    """Render the index page."""
    return jade.build_page("/")


jade.build(app)


if __name__ == "__main__":
    app.run(
        host="0.0.0.0",
        port=8080,
        debug=True,
    )
''',
    ".jade-config.yaml": """
# Add global variables to the context of all templates
site:
  name: My Site
""",
    "templates/blog.html": """
<!DOCTYPE html>
<html lang="en">

{% include 'include/head.html' %}

{% include 'include/header.html' %}

<body>
  {{content|safe}}
  {% include 'include/post-tiles.html' %}
</body>

</html>
""",
    "templates/default.html": """
<!DOCTYPE html>
<html lang="en">

{% include 'include/head.html' %}

{% include 'include/header.html' %}

<body>
  {{content|safe}}
</body>

</html>
""",
    "templates/include/head.html": """
<head>
  <meta charset="UTF-8">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>{{page.title}} | {{site.name}} </title>
  <link rel="icon" href="/static/favicon.ico" type="image/x-icon" />
  <link rel="stylesheet" href="/static/main.css" />
</head>
""",
    "templates/include/header.html": """
<header>
  <ul>
    <li><a href="/">Home</a></li>
    <li><a href="/blog">Blog</a></li>
  </ul>
  <hr>
</header>
""",
    "templates/include/post-tiles.html": """
<div>
  <br>
  {% for p in pages.blog | sort(attribute='date', reverse=True) %}
  <a href="{{p.permalink}}">
    <h2>{{p.title}}</h2>
    <p>{{p.date}}</p>
  </a>
  {% endfor %}
</div>
""",
    "static/main.css": """
:root {
  --color-dark: rgb(36, 36, 36);
  --color-light: rgb(210, 210, 210);
  --color-code-bg: rgb(50, 50, 50);
  --color-code-text: rgb(225, 225, 225);
}

* {
  padding: 0;
  margin: 0;
  font-family: sans-serif;
}

header {
  padding: 1rem;
  margin: auto;
}

header ul {
  display: flex;
  justify-content: center;
  align-items: center;
  list-style-type: none;
  padding-bottom: 1rem;
}

header li {
  display: inline;
  margin: 0 1rem;
  padding: 0.5rem;
}

header li:hover {
  text-decoration: underline;
}

body {
  background-color: var(--color-dark);
  color: var(--color-light);
  margin: auto;
  margin-top: 3rem;
  margin-bottom: 3rem;
  max-width: 70%;
}

body img {
  max-height: 500px;
  margin: auto;
  display: block;
}

hr {
  border: 0;
  height: 1px;
  background-image: linear-gradient(to right, rgba(0, 0, 0, 0), rgba(255, 255, 255, 0.75), rgba(0, 0, 0, 0));
}

h1 {
  font-size: 2rem;
  font-weight: 400;
  margin: 1rem 0;
}

h2 {
  font-size: 1.5rem;
  font-weight: 400;
  margin: 1rem 0;
}

h3 {
  font-size: 1.25rem;
  font-weight: 400;
  margin: 1rem 0;
}

h4 {
  font-size: 1rem;
  font-weight: 400;
  margin: 1rem 0;
}

h5 {
  font-size: 0.75rem;
  font-weight: 400;
  margin: 1rem 0;
}

h6 {
  font-size: 0.5rem;
  font-weight: 400;
  margin: 1rem 0;
}

p {
  font-size: 1rem;
  font-weight: 400;
  margin: 1rem 0;
}

a {
  color: var(--color-light);
  text-decoration: none;
}

a:hover {
  text-decoration: underline;
}

ul {
  list-style-type: circle;
}

li {
  font-size: 1rem;
  font-weight: 400;
  margin: 1rem 1rem 1rem 2rem;
}

code {
  background-color: var(--color-code-bg);
  color: var(--color-code-text);
  padding: 0.25rem;
  border-radius: 0.25rem;
  font-family: monospace;
}

pre {
  background-color: var(--color-code-bg);
  color: var(--color-code-text);
  padding: 0.5rem;
  border-radius: 0.25rem;
  font-family: monospace;
}

blockquote {
  border-left: 0.25rem solid var(--color-light);
  padding: 0.5rem;
  margin: 1rem 0;
}

table {
  border-collapse: collapse;
  width: 100%;
}

th {
  background-color: var(--color-light);
  color: var(--color-dark);
  padding: 0.5rem;
  text-align: left;
}

td {
  padding: 0.5rem;
  text-align: left;
}
""",
    "content/home.md": """
---
permalink: /
template: default
title: Welcome to Jade!
---
# Hello!

Welcome to the Jade markdown renderer.

To get started, try editing the `content/home.md` file.

At the top of this file, you will see some `frontmatter`. This is `yaml` formatted content that is passed to the builder, and the template.

While not required, the following `frontmatter` is recommended for each of your pages.

* `permalink` - Where on the site you want the page to register. Defaults to the file's path within the `content` directory, without the file extension.
* `template` - the name of the template in your `templates` directory to render the content into. Defaults to `default`.

## Reserved names

* `page` - You can access the page frontmatter in your templates through the `page` variable. For example, the title for this page would be accessed with `page.title`.

* `pages` - The `pages` variable contains all subdirectories of the `content` directory. This can be useful for things such as rendering a blog home page, with a link to each post in a directory. Note that nested directories are not currently supported.

* `site` - You can also access site parameters, such as the site name through the `site` variable in all of your templates. Any parameters that you need globally should be added to the `.jade-config.yaml` file, under the site key. Try updating the `name` key under `site` in the config file.
""",
    "content/blog.md": """
---
permalink: /blog
template: blog
title: Blog
---
# Welcome to my blog!

Read my articles below...
""",
    "content/blog/first-article.md": """
---
permalink: blog/my-first-article
template: default
title: My First Article
date: 2023-01-27
---
# My very first artcile!

I wrote some content here, and added some images...

![Puppy!](https://upload.wikimedia.org/wikipedia/commons/6/6e/Golde33443.jpg)
""",
    "content/blog/second-article.md": """
---
permalink: blog/my-second-article
template: default
title: My Second Article
date: 2023-01-28
---
# Another post!
Here is some more content...
""",
}



"""Return routes for each page in the content directory."""
from __future__ import annotations

import pathlib
from typing import Dict
from typing import List
from typing import Union

import flask

from jade.config import get_config
from jade.page import Page

PATHS = {}


def build(app: flask.Flask) -> None:
    """
    Build all md pages in the content directory or subdirectories.

    Params
    ------
    app: :class:`Flask`
        The Flask app to add the routes to.
    """
    files = pathlib.Path("content").rglob("*.md")

    for file in files:
        page = Page.from_path(file)
        route = _build_url(page, file)

        PATHS[route] = page

        full_route = f"{route}<path:path>"

        try:
            app.route(full_route, methods=["GET"])(build_page)
        except AssertionError:
            pass


def build_page(path: str, **kwargs) -> str:
    """
    Build a page from the content at the given path.

    Params
    ------
    path: :class:`str`
        The path to the content.

    Returns
    -------
    :class:`str` - The rendered HTML.
    """
    if path == "/":
        path = ""
    page: Union[Page, None] = PATHS.get(f"/{path}")

    if page is None:
        flask.abort(404)

    return flask.render_template(
        f"{page.frontmatter.get('template', 'default')}.html",
        content=page.render_content(**kwargs),
        page=page.frontmatter,
        **_get_files(),
        **get_config(),
        **kwargs,
    )


def _build_url(page: Page, file: pathlib.Path) -> str:
    """
    Build a URL for the given page.

    Params
    ------
    page: :class:`Page`
        The page to build the URL for.
    file: :class:`pathlib.Path`
        The path to the file.

    Returns
    -------
    :class:`str` - The URL.
    """
    route_name = str(file).split("content")[1].replace(".md", "")
    route = page.frontmatter.get("permalink", route_name)

    if not route.startswith("/"):
        return f"/{route}"
    return route


def _get_files() -> Dict[str, Union[Dict[str, List[Page]], List[Page]]]:
    """
    Get all the files in the content directory.

    Returns
    -------
    :class:`Dict[str, Union[Dict[str, List[Page]], List[Page]]]` - A list of dicts in
    the format {file: Page(file)}
    """
    paths = pathlib.Path("content").rglob("*.md")
    pages = {"pages": {}}

    for path in paths:
        if path.is_file():
            pass

        directory = path.parent

        if directory.name not in pages["pages"]:
            pages["pages"][directory.name] = []

            for file in directory.iterdir():
                if file.is_file():
                    page = Page.from_path(file)
                    pages["pages"][directory.name].append(page)

    return pages



"""Access configuration variables."""
from __future__ import annotations

import typing

import yaml


def get_config() -> typing.Dict[str, typing.Any]:
    """Get the configuration variables."""
    try:
        with open(".jade-config.yaml") as file:
            return yaml.safe_load(file)
    except Exception:
        return {"site": {"name": "My Site"}}



"""Setup file for Python lib."""
import pathlib

from setuptools import find_packages
from setuptools import setup

__name__ = "jade-md"
__version__ = "0.0.4"
__description__ = "Jade - A Python library for rendering markdown to HTML for Flask."

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name=__name__,
    version=__version__,
    description=__description__,
    url="https://github.com/sam-kenney/jade",
    author="Sam Kenney",
    author_email="sam.kenney@me.com",
    license="MIT",
    long_description=long_description,
    long_description_content_type="text/markdown",
    platforms=[],
    install_requires=[
        "flask==2.2.2",
        "jinja2==3.1.2",
        "markdown==3.4.1",
        "pyyaml==6.0",
    ],
    packages=["jade"] + ["jade." + pkg for pkg in find_packages("jade")],
)



"""Render Markdown content to HTML."""
from __future__ import annotations

import typing

import jinja2
import markdown
import yaml

import jade.path as path


class Page:
    """Represents a page of content."""

    def __init__(self, frontmatter: typing.Dict[str, str], content: str) -> Page:
        """
        Representation of a Markdown page.

        Splits the frontmatter from the content and adds the frontmatter as
        attributes to the Page object.

        Params
        ------
        frontmatter: :class:`Dict[str, str]`
            The frontmatter yaml at the top of the page.

        content: :class:`str`
            The page content as Markdown.
        """
        self.frontmatter = frontmatter
        self.content = content
        for key, value in frontmatter.items():
            setattr(self, key, value)

    def __repr__(self) -> str:
        """Return a string representation of this Page."""
        return f"Page({self.frontmatter}, {self.content})"

    @classmethod
    def from_path(cls, __path: str, /) -> Page:
        """Create a Page from the content at the given path."""
        return cls.from_markdown(cls.open_content(__path))

    @classmethod
    def from_markdown(cls, md: str) -> Page:
        """Create a Page from the given Markdown content."""
        try:
            frontmatter, content = md.split("---", 2)[1:]
            frontmatter = yaml.safe_load(frontmatter)

            return cls(frontmatter, content)
        except ValueError:
            return cls({}, md)

    @staticmethod
    def open_content(__path: str, /) -> str:
        """Open the content at the given path and return it as a string."""
        __path = path.path(__path)
        if not path.exists(__path):
            raise FileNotFoundError(f"File {__path} does not exist.")
        with open(__path) as file:
            return file.read()

    def render_content(self, **kwargs) -> str:
        """Render the content of this page to HTML."""
        html = markdown.markdown(self.content)
        return jinja2.Template(html).render(**kwargs)
